#ifndef GENERATORMODEL_H
#define GENERATORMODEL_H

#include <deque>

namespace ThesisGenerator
{

/**
 * Thesis generator model.
 * @author Máté Cserép
 */
class GeneratorModel
{
public:
    /**
     * Creates thesis generator model.
     * @param questionCount Number of questions.
     * @param periodCount  Length of period.
     */
    GeneratorModel(int questionCount, int periodCount);

    /**
     * Generates a new valid thesis number.
     * @return The generated thesis number.
     */
    int generate();

    /**
     * Takes the last generated thesis number.
     */
    void take();
    
    /**
     * Check whether a thesis number is taken.
     * @param number The thesis number to check.
     * @return True if the thesis is taken; otherwise, false.
     */
    bool isTaken(int number);

    int questionCount() { return _questionCount; }

    int periodCount() { return _periodCount; }

    int questionNumber() { return _questionNumber; }

    void setQuestionCount(int value);

    void setPeriodCount(int value);

private:
    int _questionCount;
    int _periodCount;
    int _questionNumber;
    std::deque<int> _history;
};

}

#endif // GENERATORMODEL_H
