using System;
using System.Linq;
using Xunit;

namespace ThesisGenerator.Test
{
    public class ModelTest
    {
        [Fact]
        public void AttributeTest()
        {
            Model model = new Model(10, 5);
            Assert.Equal(10, model.ThesisCount);
            Assert.Equal(5, model.RoomSize);

            model.Next();
            Assert.Equal(10, model.ThesisCount);
            Assert.Equal(5, model.RoomSize);
        }

        [Fact]
        public void LastTakenTest()
        {
            Model model = new Model(10, 5);
            Int32 taken = model.Next();
            Assert.Equal(taken, model.LastTaken);
            taken = model.Next();
            Assert.Equal(taken, model.LastTaken);
        }

        [Fact]
        public void TakenTest()
        {
            Model model = new Model(10, 5);
            Int32[] taken = new Int32[4];
            for (Int32 i = 0; i < taken.Length; ++i)
                taken[i] = model.Next();
            for (Int32 i = 0; i < taken.Length; ++i)
                Assert.True(model.IsTaken(taken[i]));

            model.Next();
            Assert.Equal(model.IsTaken(taken[0]), model.LastTaken == taken[0]);
        }

        [Fact]
        public void JustEnoughThesisTest()
        {
            Model model = new Model(10, 10);
            Int32[] taken1 = new Int32[10];
            for (Int32 i = 0; i < taken1.Length; ++i)
                taken1[i] = model.Next();

            Int32[] taken2 = new Int32[10];
            for (Int32 i = 0; i < taken2.Length; ++i)
                taken2[i] = model.Next();

            Assert.True(taken1.SequenceEqual(taken2));
        }

        [Fact]
        public void ZeroThesisTest()
        {
            Assert.Throws<ArgumentException>(() =>
            {
                Model model = new Model(0, 5);
            });
        }

        [Fact]
        public void NegativeThesisTest()
        {
            Assert.Throws<ArgumentException>(() =>
            {
                Model model = new Model(-1, 5);
            });
        }

        [Fact]
        public void NotEnoughThesisTest()
        {
            Assert.Throws<ArgumentException>(() =>
            {
                Model model = new Model(10, 11);
            });
        }
    }
}
