#pragma once

#include <gtest/gtest.h>
#include <gmock/gmock-matchers.h>

#include "../generatormodel.h"

using namespace testing;
using namespace Model;

TEST(GeneratorModelTest, Take)
{
    GeneratorModel model(10, 1);

    model.generate();
    int number = model.questionNumber();
    EXPECT_FALSE(model.isTaken(number));

    model.take(number);
    EXPECT_TRUE(model.isTaken(number));

    model.generate();
    number = model.questionNumber();
    EXPECT_FALSE(model.isTaken(number));
}

TEST(GeneratorModelTest, Retake)
{
    GeneratorModel model(10, 1);

    model.generate();
    int number = model.questionNumber();
    model.take(number);

    EXPECT_THROW(model.take(number), std::logic_error);
}

TEST(GeneratorModelTest, JustEnoughThesis)
{
    GeneratorModel model(10, 9);

    std::vector<int> taken1(10);
    for (int i = 0; i < taken1.size(); ++i)
    {
        model.generate();
        model.questionNumber();
        model.take(model.questionNumber());
        taken1[i] = model.questionNumber();
    }

    std::vector<int> taken2(10);
    for (int i = 0; i < taken2.size(); ++i)
    {
        model.generate();
        model.questionNumber();
        model.take(model.questionNumber());
        taken2[i] = model.questionNumber();
    }

    EXPECT_EQ(taken1, taken2);
}

TEST(GeneratorModelTest, ZeroThesis)
{
    EXPECT_THROW(GeneratorModel(0, 5), std::invalid_argument);
}

TEST(GeneratorModelTest, NegativeThesis)
{
    EXPECT_THROW(GeneratorModel(-1, 5), std::invalid_argument);
}

TEST(GeneratorModelTest, NotEnoughThesis)
{
    EXPECT_THROW(GeneratorModel(10, 11), std::invalid_argument);
}
