#include <vector>
#include <stdexcept>

#include <QtTest>

#include "../generatormodel.h"

using namespace Model;

class GeneratorModelTest : public QObject
{
    Q_OBJECT

private slots:
    void testTake();
    void testRetake();
    void testJustEnoughThesis();
    void testZeroThesis();
    void testNegativeThesis();
    void testNotEnoughThesis();
};

void GeneratorModelTest::testTake()
{
    GeneratorModel model(10, 1);

    model.generate();
    int number = model.questionNumber();
    QVERIFY(!model.isTaken(number));

    model.take(number);
    QVERIFY(model.isTaken(number));

    model.generate();
    number = model.questionNumber();
    QVERIFY(!model.isTaken(number));
}

void GeneratorModelTest::testRetake()
{
    GeneratorModel model(10, 1);

    model.generate();
    int number = model.questionNumber();
    model.take(number);

    QVERIFY_EXCEPTION_THROWN(model.take(number), std::logic_error);
}

void GeneratorModelTest::testJustEnoughThesis()
{
    GeneratorModel model(10, 9);

    std::vector<int> taken1(10);
    for (int i = 0; i < taken1.size(); ++i)
    {
        model.generate();
        model.questionNumber();
        model.take(model.questionNumber());
        taken1[i] = model.questionNumber();
    }

    std::vector<int> taken2(10);
    for (int i = 0; i < taken2.size(); ++i)
    {
        model.generate();
        model.questionNumber();
        model.take(model.questionNumber());
        taken2[i] = model.questionNumber();
    }

    QVERIFY(taken1 == taken2);
}

void GeneratorModelTest::testZeroThesis()
{
    QVERIFY_EXCEPTION_THROWN(GeneratorModel(0, 5), std::invalid_argument);
}

void GeneratorModelTest::testNegativeThesis()
{
    QVERIFY_EXCEPTION_THROWN(GeneratorModel(-1, 5), std::invalid_argument);
}

void GeneratorModelTest::testNotEnoughThesis()
{
    QVERIFY_EXCEPTION_THROWN(GeneratorModel(10, 11), std::invalid_argument);
}

QTEST_APPLESS_MAIN(GeneratorModelTest)

#include "tst_generatormodeltest.moc"
