#include <algorithm>
#include <stdexcept>
#include <cstdlib>

#include "generatormodel.h"

namespace Model
{

GeneratorModel::GeneratorModel(int questionCount, int periodCount)
{
    setQuestionCount(questionCount);
    setPeriodCount(periodCount);
}

void GeneratorModel::generate()
{
   _questionNumber = 1 + (rand() % _questionCount);
   while (std::find(_history.begin(), _history.end(), _questionNumber) != _history.end())
       _questionNumber = 1 + (rand() % _questionCount);

   generated(_questionNumber);
}

void GeneratorModel::take(int number)
{
    if (std::find(_history.begin(), _history.end(), number) != _history.end())
        throw std::logic_error("Given thesis already taken.");

    _history.push_back(number);
    if (_history.size() > _periodCount)
        _history.pop_front();
}

bool GeneratorModel::isTaken(int number)
{
    return std::find(_history.begin(), _history.end(), number) != _history.end();
}

void GeneratorModel::setQuestionCount(int value)
{
    if (value <= 0)
        throw std::invalid_argument("Thesis count must be a positive number.");

    _questionCount = value;
    if (_periodCount > _questionCount) {
        _periodCount = value - 1;

        _history.erase(
            std::remove_if(
                _history.begin(),
                _history.end(),
                [value](int n) {return n > value; }),
            _history.end());

        while (_history.size() > _periodCount)
            _history.pop_front();
    }
}

void GeneratorModel::setPeriodCount(int value)
{
    if (value < 0)
        throw std::invalid_argument("Period count must be a non-negative number.");

    if (value >= _questionCount)
        throw std::invalid_argument("Period count must be smaller than the thesis count.");

    _periodCount = value;
    while(_history.size() > _periodCount)
        _history.pop_front();
}

}
