#ifndef GENERATORMODEL_H
#define GENERATORMODEL_H

#include <deque>

#include <QObject>

namespace Model
{

/**
 * Thesis generator model.
 * @author Máté Cserép
 */
class GeneratorModel : public QObject
{
    Q_OBJECT

public:
    /**
     * Creates thesis generator model.
     * @param questionCount Number of questions.
     * @param periodCount  Length of period.
     */
    GeneratorModel(int questionCount, int periodCount);

    /**
     * Generates a new valid thesis number.
     */
    void generate();

    /**
     * Takes a thesis number.
     * @param number The thesis number to take.
     */
    void take(int number);

    /**
     * Check whether a thesis number is taken.
     * @param number The thesis number to check.
     * @return True if the thesis is taken; otherwise, false.
     */
    bool isTaken(int number);

    int questionCount() { return _questionCount; }

    int periodCount() { return _periodCount; }

    int questionNumber() { return _questionNumber; }

    void setQuestionCount(int value);

    void setPeriodCount(int value);

signals:
    /**
     * Signals that a new thesis number has been generated.
     * @param number The generated thesis number.
     */
    void generated(int number);

private:
    int _questionCount;
    int _periodCount;
    int _questionNumber;
    std::deque<int> _history;
};

}

#endif // GENERATORMODEL_H
