#include "mainwindow.h"
#include "ui_mainwindow.h"

using namespace Model;

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    // Disable resizing
    setWindowFlags(Qt::Dialog | Qt::MSWindowsFixedSizeDialogHint);
    statusBar()->setSizeGripEnabled(false);

    // Model
    model = new GeneratorModel(10, 0);
    ui->spinnerQuestion->setValue(model->questionCount());
    ui->spinnerPeriod->setValue(model->periodCount());
    connect(model, SIGNAL(generated(int)), this, SLOT(onGenerated(int)));

    // Timer
    timer = new QTimer(this);
    connect(timer, SIGNAL(timeout()), this, SLOT(onTick()));

    // UI signals
    connect(ui->btnStartStop, SIGNAL(clicked()), this, SLOT(onStartStop()));
    connect(ui->spinnerQuestion, SIGNAL(valueChanged(int)), this, SLOT(onQuestionCountChanged(int)));
    connect(ui->spinnerPeriod, SIGNAL(valueChanged(int)), this, SLOT(onPeriodCountChanged(int)));
}

MainWindow::~MainWindow()
{
    delete ui;
    delete model;
    delete timer;
}

void MainWindow::onGenerated(int number)
{
    thesisNumber = number;
    ui->txtNumber->setText(std::to_string(number).c_str());
}

void MainWindow::onStartStop()
{
    if (timer->isActive())
    {
        timer->stop();
        ui->btnStartStop->setText("START");
    }
    else
    {
        timer->start(10);
        ui->btnStartStop->setText("STOP");

        model->take(thesisNumber);
    }
}

void MainWindow::onTick()
{
    model->generate();
}

void MainWindow::onQuestionCountChanged(int value)
{
    model->setQuestionCount(value);
    ui->spinnerPeriod->setValue(model->periodCount());
}

void MainWindow::onPeriodCountChanged(int value)
{
    model->setPeriodCount(value);
}
